
/*
Eagles Bulletin Board System
Copyright (C) 1994, Ray Rocker, rrrocker@rock.b11.ingr.com
                                rock@seabass.st.usm.edu
                                72673.2105@compuserve.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "client.h"
#include <time.h>
#include <ctype.h>

/* The currently selected board */
NAME currboard;

/* stuff in c_users.c for the permission mask setting */
extern LONG SetPermMenu __P((LONG));

CheckBoardName(bname)
char *bname;
{
  BOARD buf;
  if (bname[0] == '\0') return 1;
  if (!is_valid_boardname(bname)) return 3;
  if (bbs_get_board(bname, &buf) != S_NOTFOUND) return 2; 
  return 0;
}

/*ARGSUSED*/
AllBoardsFunc(indx, board, info)
int indx;
BOARD *board;
struct enum_info *info;
{
  extern int lang;

  if (info->topline == info->currline) {
    move(info->topline-1, 0);
    standout();
    if (lang)
      prints(" %-16s   %-58s\n", "̸", "弳");
    else
      prints(" %-16s   %-58s\n", "Name", "Description");
    standend();
  }

  if (lang && *board->kdescription)
    prints("%c%-15s %c%c %s\n", 
	 BITISSET(board->flags, BOARD_ZAPPED) ? '*' : ' ',
	 board->name, 
	 board->readmask ? 'R' : ' ',
	 board->postmask ? 'P' : ' ',
	 board->kdescription);
  else
    prints("%c%-15s %c%c %s\n", 
	 BITISSET(board->flags, BOARD_ZAPPED) ? '*' : ' ',
	 board->name, 
	 board->readmask ? 'R' : ' ',
	 board->postmask ? 'P' : ' ',
	 board->description);

  info->currline++;
  info->count++;

  if (info->currline > info->bottomline) {
    int ch;
    standout();
    c_msg(INFO_MORE, FALSE);
    standend();
    clrtoeol();
    while((ch = igetch()) != EOF) {
      if(ch == '\n' || ch == '\r' || ch == ' ')
	break;
      if(toupper(ch) == 'Q') {
	move(info->currline, 0);
	clrtoeol();
	return ENUM_QUIT;
      }
      else bell();
    }
    info->currline = info->topline;
  }
  return S_OK;
}

Boards()
{
  struct enum_info info;
  extern int lang;

  info.count = 0;
  info.topline = info.currline = 4;
  info.bottomline = t_lines-2;
  move(2,0);
  clrtobot();    
  bbs_enum_boards(t_lines-5, 0, BE_ALL, AllBoardsFunc, &info);
  clrtobot();
  move(t_lines-1, 0);
  if (lang)
    prints("%d %s ֽϴ\n", info.count, info.count==1?"尡":"");
  else
    prints("%d %s displayed\n", info.count, info.count==1?"board":"boards");
  return PARTUPDATE;
}

/*ARGSUSED*/
BoardCountsFunc(indx, board, info)
int indx;
BOARD *board;
struct enum_info *info;
{
  extern int lang;
  extern char *Ctime __P((time_t *));

  if (info->topline == info->currline) {
    move(info->topline-1, 0);
    if (lang)
      prints(" %-16s   %6s %6s   %-30s\n",
	"", "հ", "", " ");
    else
      prints(" %-16s   %6s %6s   %-30s\n",
	"Board", "Total", "Unread", "Last Post");
  }

  prints("%c%-16s   %6d %6d   %s\n", 
	 BITISSET(board->flags, BOARD_ZAPPED) ? '*' : ' ',
	 board->name, 
	 board->totalposts,
       	 board->newposts,
	 (board->lastpost == 0 ? "" : Ctime((time_t *)&board->lastpost)));

  info->currline++;
  info->count++;
  info->totals[0] += board->totalposts;
  info->totals[1] += board->newposts;

  if (info->currline > info->bottomline) {
    int ch;
    standout();
    c_msg(INFO_MORE, FALSE);
    standend();
    clrtoeol();
    while((ch = igetch()) != EOF) {
      if(ch == '\n' || ch == '\r' || ch == ' ')
	break;
      if(toupper(ch) == 'Q') {
	move(info->currline, 0);
	clrtoeol();
	return ENUM_QUIT;
      }
      else bell();
    }
    info->currline = info->topline;
  }
  return S_OK;
}

BoardCounts()
{
  struct enum_info info;
  SHORT flags;
  char ans[9];
  extern int lang;

  info.count = 0;
  info.topline = info.currline = 4;
  info.bottomline = t_lines-2;
  info.totals[0] = info.totals[1] = 0;
    
  move(2,0);
  clrtobot();
  if (lang) {
    if (getdata(3, 0, "(U)-ܽŰ  ͸, (Z)-ܽŲ ͸, or (A)-? [U]: ",
          ans, sizeof ans, DOECHO, 1) == -1) return FULLUPDATE;
  } else {
    if (getdata(3, 0, "(U)nzapped only, (Z)apped only, or (A)ll? [U]: ",
          ans, sizeof ans, DOECHO, 1) == -1) return FULLUPDATE;
  }

  if (*ans == 'Z' || *ans == 'z') flags = BE_ZAPPED;
  else if (*ans == 'A' || *ans == 'a') flags = BE_ALL;
  else flags = BE_UNZAPPED;

  flags |= BE_DO_COUNTS;
  bbs_enum_boards(t_lines-5, 0, flags, BoardCountsFunc, &info);
  
  clrtobot();
  move(info.currline, 0);

  if (lang)
    prints(" %-16s   %6d %6d\n", "*** հ ***", 
         info.totals[0], info.totals[1]);
  else
    prints(" %-16s   %6d %6d\n", "*** TOTALS ***", 
         info.totals[0], info.totals[1]);

  return PARTUPDATE;
}

SelectBoard()
{
  NAME newboard;
  NAMELIST boardlist = NULL;
  extern int lang;

  move(2,0);
  clrtobot();
  move(3,0);
  bbs_boardnames(&boardlist);
  if (boardlist->size == 0) {
    c_msg(ERR_NOTHING, TRUE);
    free_namelist(&boardlist);
    pressreturn();
    return FULLUPDATE;
  }
  if (lang)
    namecomplete(boardlist, "̸?: ", newboard);
  else
    namecomplete(boardlist, "Enter a board name: ", newboard);
  if (newboard[0] == '\0') return FULLUPDATE;
  else if (!is_in_namelist(boardlist, newboard)) {
    move(4,0);
    c_msg(ERR_NOT_EXIST, TRUE);
    free_namelist(&boardlist);
    pressreturn();
    return FULLUPDATE;
  }
  strcpy(currboard, newboard);
  free_namelist(&boardlist);
  return (FULLUPDATE | NEWDIRECT);
}

Zap()
{
  NAME zapboard;
  BOARD brec;
  SHORT notzapped;
  NAMELIST boardlist = NULL;
  extern int lang;

  move(2,0);
  clrtobot();
  bbs_boardnames(&boardlist);
  if (boardlist->size == 0) {
    c_msg(ERR_NOTHING, TRUE);
    free_namelist(&boardlist);
    pressreturn();
    return FULLUPDATE;
  }
  if (lang)
    namecomplete(boardlist, " : ", zapboard);
  else
    namecomplete(boardlist, "Board to zap/unzap: ", zapboard);
  if (zapboard[0] == '\0') return PARTUPDATE;
  if (!is_in_namelist(boardlist, zapboard) || 
      bbs_get_board(zapboard, &brec) != S_OK) {
    move(4,0);
    c_msg(ERR_NOT_EXIST, TRUE);
    free_namelist(&boardlist);
    pressreturn();
    return FULLUPDATE;
  }
  notzapped = !BITISSET(brec.flags, BOARD_ZAPPED);
  if (bbs_zap_board(brec.name, notzapped) == S_ILLEGAL) {
    move(3,0);
    c_msg(ERR_FAILURE, TRUE);
  }
  free_namelist(&boardlist);
  return PARTUPDATE;
}

Visit()
{
  NAME visitboard;
  char ans[4];
  NAMELIST boardlist = NULL;
  extern int lang;

  move(2,0);
  clrtobot();
  bbs_boardnames(&boardlist);
  if (boardlist->size == 0) {
    c_msg(ERR_NOTHING, TRUE);
    free_namelist(&boardlist);
    pressreturn();
    return FULLUPDATE;
  }
  add_namelist(&boardlist, "*", NULL);
  sort_list(&boardlist);

  if (lang)
    namecomplete(boardlist, " 带 (*=zap   ): ", visitboard);
  else
    namecomplete(boardlist, "Visit board (* = all unzapped boards): ", visitboard);
  remove_namelist(&boardlist, "*");
  sort_list(&boardlist);
  if (visitboard[0] == '\0') return FULLUPDATE;
  else if (visitboard[0] != '*' && !is_in_namelist(boardlist, visitboard)) {
    move(4,0);
    c_msg(ERR_NOT_EXIST, TRUE);
    free_namelist(&boardlist);
    pressreturn();
    return FULLUPDATE;
  }
  if (lang)
    getdata(3,0," (Y/N)? [N]: ", ans, sizeof(ans), DOECHO, 0);
  else
    getdata(3,0,"Are you sure (Y/N)? [N]: ", ans, sizeof(ans), DOECHO, 0);
  move(4,0);
  if (*ans != 'Y' && *ans != 'y') {
    c_msg(INFO_CANCELLED, TRUE);
    free_namelist(&boardlist);
    pressreturn();
    return FULLUPDATE;
  }

  /* this can take a while, so...*/
  c_msg(INFO_IN_PROGRESS, TRUE);
  refresh();
  if (bbs_visit_board(visitboard) == S_OK) {
    c_msg(INFO_DONE, TRUE);
  } else {
    c_msg(ERR_FAILURE, TRUE);
  }
  free_namelist(&boardlist);
  pressreturn();
  return FULLUPDATE;
}

SetBoardPerms(brec, postp)
BOARD *brec;
int postp;
{
  LONG newperms;
  extern int lang;

  move(2,0);
  clrtobot();
  if (lang)
    prints("%s   %s ̼ ϱ\n",
	 brec->name, (postp ? "":"б"));
  else
    prints("Set the %s permissions for board '%s'\n", 
	 (postp ? "post" : "read"), brec->name);
  newperms = SetPermMenu(postp ? brec->postmask : brec->readmask);	
  clear();
  if (newperms != (postp ? brec->postmask : brec->readmask)) {
    if (postp) brec->postmask = newperms;
    else brec->readmask = newperms;
    return 0;
  }
  else return -1;
}

AddBoard()
{
  BOARD board;
  int y, grok;
  int rc;
  char ans[4];
  extern int lang;

  move(2,0);
  clrtobot();
  memset(&board, 0, sizeof(BOARD));
  do {
    if (lang)
      {
        if (getdata(3,0," ̸: ",board.name,NAMELEN+1,DOECHO,1)==-1)
          return FULLUPDATE;
      }
    else
      {
        if (getdata(3,0,"New board name: ",board.name,NAMELEN+1,DOECHO,1)==-1)
          return FULLUPDATE;
      }

    grok = CheckBoardName(board.name);
    switch (grok) {
    case 0:
	break;
    case 1:
	c_msg(ERR_NO_NAME, TRUE);
	break;
    case 2:
	c_msg(ERR_EXIST, TRUE);
	break;
    case 3:
	c_msg(ERR_INVALID_NAME, TRUE);
	break;
    default:
	c_msg(ERR_UNKNOWN, TRUE);
	break;
    }
  } while (grok);

  if (lang)
    {
      getdata(4,0,"弳 (): ", board.description, TITLELEN+1, DOECHO, 0);
      getdata(5,0,"弳 (ѱ): ", board.kdescription, TITLELEN+1, DOECHO, 0);
      getdata(6,0,"б  (Y/N)? [N]: ",ans,sizeof(ans),DOECHO,0);
    }
  else
    {
      getdata(4,0,"Board description (English): ", board.description, TITLELEN+1, DOECHO, 0);
      getdata(5,0,"Board description (Korean): ", board.kdescription, TITLELEN+1, DOECHO, 0);
      getdata(6,0,"Set read restrictions (Y/N)? [N]: ",ans,sizeof(ans),DOECHO,0);
    }
  if (*ans == 'Y' || *ans == 'y') {
    SetBoardPerms(&board, 0);
    y = 0;
  }
  else y = 7;
  if (lang)
    getdata(y,0,"۾  (Y/N)? [N]: ",ans,sizeof(ans),DOECHO,0);
  else
    getdata(y,0,"Set post restrictions (Y/N)? [N]: ",ans,sizeof(ans),DOECHO,0);
  if (*ans == 'Y' || *ans == 'y') {
    SetBoardPerms(&board, 1);
    y = 0;
  }
  else y = 8;
  if (lang)
    getdata(y,0,"  ұ (Y/N)? [N]: ",ans,sizeof(ans), DOECHO, 0);
  else
    getdata(y,0,"Add board: are you sure (Y/N)? [N]: ",ans,sizeof(ans), DOECHO, 0);
  if (*ans != 'Y' && *ans != 'y') {
    c_msg(INFO_CANCELLED, TRUE);
  } else {
    if ((rc = bbs_add_board(&board)) == S_OK)
	c_msg(INFO_ADDED, TRUE);
    else 
	c_msg(ERR_FAILURE, TRUE);
  }
  pressreturn();
  return FULLUPDATE;
}

DeleteBoard()
{
  NAME namebuf;
  char ans[4];
  NAMELIST boardlist = NULL;
  extern int lang;

  move(2,0);
  clrtobot();
  bbs_boardnames(&boardlist);
  if (boardlist->size  == 0) {
    c_msg(ERR_NOTHING, TRUE);
    free_namelist(&boardlist);
    pressreturn();
    return FULLUPDATE;
  }
  namecomplete(boardlist, "Delete which board: ", namebuf);        
  if (namebuf[0] == '\0') return FULLUPDATE;
  else if (!is_in_namelist(boardlist, namebuf)) {
    c_msg(ERR_NOT_EXIST, TRUE);
    free_namelist(&boardlist);
    pressreturn();
    return FULLUPDATE;
  }
  c_msg(INFO_DELETING, TRUE);
  if (lang) {
    getdata(5,0," (Y/N)? [N]: ",ans,sizeof(ans), DOECHO, 0);    
  } else {
    getdata(5,0,"Are you sure (Y/N)? [N]: ",ans,sizeof(ans), DOECHO, 0);    
  }
  if (ans[0] != 'Y' && ans[0] != 'y') {
    c_msg(INFO_CANCELLED, TRUE);
    free_namelist(&boardlist);
    pressreturn();
    return FULLUPDATE;
  }
  if (bbs_delete_board(namebuf) == S_OK) {
    c_msg(INFO_DELETED, TRUE);
  } else {
    c_msg(ERR_FAILURE, TRUE);
  }
  free_namelist(&boardlist);
  pressreturn();
  return FULLUPDATE;
}

ChangeBoard()
{
  NAME namebuf, oldname;
  BOARD brec;
  int y, grok;
  SHORT flags = 0;
  char ans[4];
  NAMELIST boardlist = NULL;
  extern int lang;

  move(2,0);
  clrtobot();
  bbs_boardnames(&boardlist);
  if (boardlist->size == 0) {
    c_msg(ERR_NOTHING, TRUE);
    free_namelist(&boardlist);
    pressreturn();
    return FULLUPDATE;
  }
  namecomplete(boardlist, "Change which board: ", namebuf);
  if (namebuf[0] == '\0') return FULLUPDATE;
  else if (!is_in_namelist(boardlist, namebuf)) {
    c_msg(ERR_NOT_EXIST, TRUE);
    free_namelist(&boardlist);
    pressreturn();
    return FULLUPDATE;
  }
  if (bbs_get_board(namebuf, &brec) != S_OK) {
    c_msg(ERR_FAILURE, TRUE);
    free_namelist(&boardlist);
    pressreturn();
    return FULLUPDATE;
  }
  strncpy(oldname, brec.name, NAMELEN);
  move(3,0);
  clrtobot();
  move(4,0);
  if (lang)
    {
      prints(" ̸: %s\n", brec.name);
      prints("  (): %s\n", brec.description);
      prints("  (ѱ): %s\n", brec.kdescription);
      prints(" ڸ ٲַ (M)-Ŵ  ϼ.\n");
      if (brec.readmask) prints("б  ֽϴ.\n");
      if (brec.postmask) prints("  ֽϴ.\n");
    }
  else
    {
      prints("Board name: %s\n", brec.name);
      prints("Board description (English): %s\n", brec.description);
      prints("Board description (Korean): %s\n", brec.kdescription);
      prints("Use (M)anagers to view and set board managers.\n");
      if (brec.readmask) prints("Read restrictions are in effect.\n");
      if (brec.postmask) prints("Post restrictions are in effect.\n");
    }
  do {
    if (lang)
      {
        getdata(10,0," ̸ ( ġ ϴ): ", 
	    namebuf, NAMELEN+1, DOECHO, 0);
      }
    else
      {
        getdata(10,0,"New board name (ENTER to leave unchanged): ", 
	    namebuf, NAMELEN+1, DOECHO, 0);
      }
    grok = CheckBoardName(namebuf);
    switch (grok) {
    case 0: BITSET(flags, MOD_BNAME);
      strncpy(brec.name, namebuf, sizeof(brec.name));
      break;
    case 1: grok = 0;
      break;
    case 2:
      c_msg(ERR_EXIST, TRUE);
      break;
    case 3:
      c_msg(ERR_INVALID_NAME, TRUE);
      break;
    default:
      c_msg(ERR_UNKNOWN, TRUE);
      break;
    }
  } while (grok);

  if (lang)
    getdata(11,0,"   (): ", brec.description, TITLELEN, DOECHO, 0);
  else
    getdata(11,0,"New description (English) if any: ", brec.description, TITLELEN, DOECHO, 0);

  if (brec.description[0] != '\0') BITSET(flags, MOD_BOARDDESC);

  if (lang)
    getdata(12,0,"   (ѱ): ", brec.kdescription, TITLELEN, DOECHO, 0);
  else
    getdata(12,0,"New description (Korean) if any: ", brec.kdescription, TITLELEN, DOECHO, 0);

  if (brec.kdescription[0] != '\0') BITSET(flags, MOD_BOARDKDESC);

  if (lang)
    getdata(13,0,"б  ٲܱ (Y/N)? [N]: ", ans,sizeof(ans),DOECHO,0);
  else
    getdata(13,0,"Alter read restrictions (Y/N)? [N]: ", ans,sizeof(ans),DOECHO,0);
  if (*ans == 'Y' || *ans == 'y')
    {
      if (SetBoardPerms(&brec, 0) != -1)
        BITSET(flags, MOD_READMASK);
      y = 0;
    }
   else
    {
      y = 14;
    }

  if (lang)
    getdata(y,0,"  ٲܱ (Y/N)? [N]: ", ans,sizeof(ans),DOECHO,0);
  else
    getdata(y,0,"Alter post restrictions (Y/N)? [N]: ", ans,sizeof(ans),DOECHO,0);
  if (*ans == 'Y' || *ans == 'y')
    {
      if (SetBoardPerms(&brec, 1) != -1)
        BITSET(flags, MOD_POSTMASK);
      y = 0;
    }
  else
    {
      y = 16;
    }

  if (lang)
    getdata(y,0," 带 ٲܱ (Y/N)? [N]: ",ans,sizeof(ans), DOECHO,0);
  else
    getdata(y,0,"Change board: are you sure (Y/N)? [N]: ",ans,sizeof(ans), DOECHO,0);

  if (*ans != 'Y' && *ans != 'y')
    {
      c_msg(INFO_CANCELLED, TRUE);
    }
  else
    {
      if (bbs_modify_board(oldname, &brec, flags) == S_OK)
	c_msg(INFO_CHANGED, TRUE);
      else
	c_msg(ERR_FAILURE, TRUE);
    }
  free_namelist(&boardlist);
  pressreturn();
  return FULLUPDATE;
}
